﻿const express = require('express');
const { createClient } = require('@redis/client');
const { WebSocketServer } = require('ws');
const path = require('path');
const bodyParser = require('body-parser'); // bodyParser를 사용하기 위해 추가

// Create an Express application
const app = express();
const port = 4000;

let redisPublicKey = '11270016379155155155155';
let redisClient; // Redis 클라이언트를 전역 변수로 선언

// Serve static files from the public directory
app.use(express.static(path.join(__dirname, 'public')));
app.use(bodyParser.json()); // bodyParser 추가

// favicon 경로를 수동으로 설정
app.get('/favicon.ico', (req, res) => res.status(204).end());

app.listen(port, '0.0.0.0', () => {
    console.log(`HTTP Server is running at http://0.0.0.0:${port}`);
});
const wss = new WebSocketServer({ port: 8090, host: '0.0.0.0' });

let connectedClients = [];

// Handle WebSocket connections
wss.on('connection', (ws) => {
    console.log('New WebSocket connection');
    connectedClients.push(ws);

    ws.on('close', () => {
        connectedClients = connectedClients.filter(client => client !== ws);
        console.log('WebSocket disconnected');
    });
});

async function connectToRedis() {
    if (redisPublicKey && redisPublicKey.length >= 4) {
        redisClient = createClient();
        redisClient.on('error', (err) => console.error('Redis Client Error', err));
        await redisClient.connect();
    } else {
        console.error('Cannot connect to Redis: Invalid public key');
    }
}

// Redis에 연결이 안 되어 있을 때만 연결을 시도하는 함수
async function ensureConnectedToRedis() {
    if (!redisClient || !redisClient.isOpen) {
        await connectToRedis();
    } else {
        console.log('Redis is already connected');
    }
}

async function DebugconnectToRedis() {
    await ensureConnectedToRedis();
}

// Function to fetch data from Redis list using BLPOP (blocking pop)
async function fetchDataFromRedis() {

	const currentTime = new Date().toISOString(); // 현재 시간을 ISO 형식으로 가져옴
	//console.error(`[${currentTime}] fetchDataFromRedis entered: [${redisPublicKey}]`);

    try {
        const result = await redisClient.blPop(`QCS:${redisPublicKey}`, 0); // redisPublicKey 사용
        if (result) {
            const data = JSON.parse(result.element);
            return Array.isArray(data) ? data : [data]; // Ensure data is an array
        }
    } catch (err) {
        console.error('Error fetching data from Redis:', err);
    }
    return [];
}

if(true)
{
	// Polling Redis every second and sending data to WebSocket clients
	setInterval(async () => {
		if (redisClient && redisPublicKey && redisPublicKey.length >= 4) { // Redis 클라이언트 연결 여부 체크
			const processedData = await fetchDataFromRedis();
			console.log('Sending data:', processedData); // Log the data being sent
			connectedClients.forEach((client) => {
				if (client.readyState === client.OPEN) {
					client.send(JSON.stringify(processedData));
				}
			});
		}
	}, 1000); // 1 second interval
}

// Clean up on exit
process.on('SIGINT', async () => {
    console.log('Closing Redis client...');
    if (redisClient) {
        await redisClient.quit();
    }
    process.exit(0);
});


// Redis 연결 함수
const connectbesideredisClientRedis = () => {
    besideredisClient = createClient(); // Redis 클라이언트 생성
    besideredisClient.on('connect', () => console.log('Connected to Redis for Search!!.'));
    besideredisClient.on('error', (err) => console.error('Redis error:', err));
    besideredisClient.connect(); // Redis 연결

    DebugconnectToRedis().catch(console.error);
};

// Redis 연결 초기화
connectbesideredisClientRedis();

// Redis Public Key 설정 엔드포인트
app.post('/set-public-key', async (req, res) => {
    const { publicKey } = req.body;

    if (publicKey) {
        redisPublicKey = publicKey; // publicKey 업데이트
        console.log('Redis Public Key updated to:', redisPublicKey);

		DebugconnectToRedis().catch(console.error);

        res.json({ message: 'Public Key updated successfully.' });
    } else {
        res.status(400).json({ error: 'Public Key is required.' });
    }
});

app.post('/send', async (req, res) => {
    const sendData = req.body;
    console.log('Received data to send:', sendData);

    // sender 필드 출력
    console.log('Sender:', sendData.sender);
    
    try {
        const jsonText = JSON.stringify(sendData);

        // 키의 데이터 유형 확인
        //const recvkey = 'CLK:' + sendData.sender;
		let recvkey = 'CLK';
		recvkey = recvkey.replace(/\./g, ''); // Remove all '.' characters
        const type = await besideredisClient.type(recvkey);
        
        if (type !== 'none' && type !== 'list') {
            // 키가 존재하고 리스트가 아닌 경우 삭제
            await besideredisClient.del(recvkey);
        }

        // lPush로 데이터 전송
        await besideredisClient.lPush(recvkey, jsonText);
        console.log('Data sent to Redis:', jsonText);
        res.json({ message: 'Data sent successfully.' });
    } catch (error) {
        console.error('Failed to send data to Redis:', error);
        res.status(500).json({ error: 'Failed to send data.' });
    }
});

// 데이터 수신
app.get('/receive', async (req, res) => {
    console.log('Request received to fetch data from Redis');
    
    try {
		let sendkey = 'QCK:' + redisPublicKey;
		sendkey = sendkey.replace(/\./g, ''); // Remove all '.' characters
        // Redis 클라이언트에서 blPop 명령 호출 (블로킹 팝)
        const result = await besideredisClient.blPop(sendkey, 0); // 0 means it will block until data is available
        
        if (result) {
            const data = JSON.parse(result.element); // result.element는 값
			console.log('Request received to fetch data from Redis:', data);
            return res.json(data); // 데이터를 JSON으로 반환
        }
    } catch (err) {
        console.error('Error fetching data from Redis:', err);
        return res.status(500).json({ error: 'Failed to fetch data.' });
    }
    
    return res.status(404).json({ message: 'No data found.' });
});

